from functools import partial

import tensorflow as tf


def augment_image(
        brightness_delta=0.05,
        contrast_factor=[0.7, 1.3],
        saturation=[0.6, 1.6]):
    """ Helper function used for augmentation of images in the dataset.
      Args:
        brightness_delta: maximum value for randomly assigning brightness of the image.
        contrast_factor: list / tuple of minimum and maximum value of factor to set random contrast.
                          None, if not to be used.
        saturation: list / tuple of minimum and maximum value of factor to set random saturation.
                    None, if not to be used.
      Returns:
        tf.data.Dataset mappable function for image augmentation
    """

    def augment_fn(data, label, *args, **kwargs):
        # Augmenting data (~ 80%)

        def augment_steps_fn(data, label):
            # Randomly rotating image (~50%)
            def rotate_fn(data, label):
                times = tf.random.uniform(minval=1, maxval=4, dtype=tf.int32, shape=[])
                return (tf.image.rot90(data, times),
                        tf.image.rot90(label, times))

            data, label = tf.cond(
                tf.less_equal(tf.random.uniform([]), 0.5),
                lambda: rotate_fn(data, label),
                lambda: (data, label))

            # Randomly flipping image (~50%)
            def flip_fn(data, label):
                return (tf.image.flip_left_right(data),
                        tf.image.flip_left_right(label))

            data, label = tf.cond(
                tf.less_equal(tf.random.uniform([]), 0.5),
                lambda: flip_fn(data, label),
                lambda: (data, label))

            # Randomly setting brightness of image (~50%)
            # def brightness_fn(data, label):
            #     delta = tf.random.uniform(minval=0, maxval=brightness_delta, dtype=tf.float32, shape=[])
            #     return (tf.image.adjust_brightness(data, delta=delta),
            #             tf.image.adjust_brightness(label, delta=delta))
            #
            # data, label = tf.cond(
            #     tf.less_equal(tf.random.uniform([]), 0.5),
            #     lambda: brightness_fn(data, label),
            #     lambda: (data, label))
            #
            # # Randomly setting constrast (~50%)
            # def contrast_fn(data, label):
            #     factor = tf.random.uniform(
            #         minval=contrast_factor[0],
            #         maxval=contrast_factor[1],
            #         dtype=tf.float32, shape=[])
            #     return (tf.image.adjust_contrast(data, factor),
            #             tf.image.adjust_contrast(label, factor))
            #
            # if contrast_factor:
            #     data, label = tf.cond(
            #         tf.less_equal(tf.random.uniform([]), 0.5),
            #         lambda: contrast_fn(data, label),
            #         lambda: (data, label))
            #
            # # Randomly setting saturation(~50%)
            # def saturation_fn(data, label):
            #     factor = tf.random.uniform(
            #         minval=saturation[0],
            #         maxval=saturation[1],
            #         dtype=tf.float32,
            #         shape=[])
            #     return (tf.image.adjust_saturation(data, factor),
            #             tf.image.adjust_saturation(label, factor))
            #
            # if saturation:
            #     data, label = tf.cond(
            #         tf.less_equal(tf.random.uniform([]), 0.5),
            #         lambda: saturation_fn(data, label),
            #         lambda: (data, label))

            return data, label

        # Randomly returning unchanged data (~20%)
        return tf.cond(
            tf.less_equal(tf.random.uniform([]), 0.2),
            lambda: (data, label),
            partial(augment_steps_fn, data, label))

    return augment_fn
